function b = BSplineEval(varargin)
%
% yy = BSplineEval(j,k,t,xx)
%
% j-th B-spline of order k on knots t evaluated at xx
%
% b = BSplineEval(k,t,xx)
%
% b is the cell array of all the length(t)-k B-splines of order k on
% knots t evaluated at xx
%
if (nargin == 4)
  j = varargin{1};
  k = varargin{2};
  t = varargin{3};
  xx = varargin{4};
  n = length(t);
  if (j > n-k)
    error('j has to be smaller or equal to length(t)-k')
  end
else
  k = varargin{1};
  t = varargin{2};
  xx = varargin{3};
  n = length(t);
end
for i = 1:n-1
  b{i} = zeros(size(xx));
  b{i}(t(i) <= xx & xx < t(i+1)) = 1;
end
for h = 2:k
  for i = 1:n-h
    val = t(i+h-1) == t(i);
    b{i} = (1-val) * (xx-t(i))/(t(i+h-1)-t(i)+val).*b{i};
    val = t(i+h) == t(i+1);
    b{i} = b{i} + (1-val) * (t(i+h)-xx)/(t(i+h)-t(i+1)+val).*b{i+1};
  end
end
b{n-k}(xx == t(end)) = 1; % continuity condition
b = b(1:n-k); % this line was missing
if (nargin == 4)
  b = b{j};
end
%!demo
%! n = 4;
%! t = linspace(0,n,n+1);
%! k = 3;
%! t = [zeros(1,k-1),t,n*ones(1,k-1)];
%! xx = linspace(0,n);
%! plot(xx,BSplineEval(1,k,t,xx),xx,BSplineEval(2,k,t,xx),...
%! xx,BSplineEval(3,k,t,xx),xx,BSplineEval(4,k,t,xx),...
%! xx,BSplineEval(5,k,t,xx),xx,BSplineEval(6,k,t,xx))
%! title('B-splines of order 3')